package com.yundong.third.server.impl;

import cn.hutool.core.util.IdUtil;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.yundong.base.result.Result;
import com.yundong.gateway.register.annotation.YdcGateway;
import com.yundong.third.easyExcel.listener.ExcelListener;
import com.yundong.third.entity.IssuesListDetail;
import com.yundong.third.entity.IssuesListDetailExcel;
import com.yundong.third.mapper.IssuesListDeatilMapper;
import com.yundong.third.server.IssuesListDetailServer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.sql.Date;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
@org.apache.dubbo.config.annotation.Service(version = "${dubbo.provider.version}", group = "${dubbo.provider.group}")
public class IssuesListDetailServerImpl implements IssuesListDetailServer {
    @Autowired
    private IssuesListDeatilMapper issuesListDeatilMapper;

    /**
     * excel导入
     *
     * @param excel     excel文件
     * @param projectId 项目id
     * @param userId    当前登录人id
     * @return
     */
    @YdcGateway(name = "monitorCounter-issuesList", url = "third/monitor/issuesListDetail/importExcel.json")
    @Override
    public Result<String> importExcel(File excel, String projectId, String userId) {
        if (excel == null) {
            return Result.fail("文件不能为空");
        }
        if (excel.getName().endsWith(".xlsx")) {
            return Result.fail("请导入xlsx格式的文件");
        }
        try {
            //解析
            ExcelListener excelListener = new ExcelListener();
            EasyExcelFactory.read(excel, IssuesListDetailExcel.class, excelListener).excelType(ExcelTypeEnum.XLSX).sheet().doRead();
            List<IssuesListDetail> successData = excelListener.getSuccessData();

            //查询
            List<IssuesListDetail> issuesListDetails = issuesListDeatilMapper.selectList(
                    new LambdaQueryWrapper<IssuesListDetail>()
                            .eq(IssuesListDetail::getIsDeleted, 0)
                            .eq(IssuesListDetail::getPROJECTID, projectId));
            String projectName = issuesListDetails.get(NumberUtils.INTEGER_ZERO).getPROJECTNAME();
            String tenantId = issuesListDetails.get(NumberUtils.INTEGER_ZERO).getTenantId();
            //过滤
            List<IssuesListDetail> importDataList = successData.stream()
                    .filter(item1 -> issuesListDetails.stream().noneMatch(item2 -> item2.getDISCOVERYTIME().equals(item1.getDISCOVERYTIME())
                            && item2.getDESCRIBETEXT().equals(item1.getDESCRIBETEXT())))
                    .collect(Collectors.toList());

            //赋值导入
            Date date = new Date(System.currentTimeMillis());
            for (IssuesListDetail issuesListDetail : importDataList) {
                issuesListDetail.setId(IdUtil.simpleUUID());
                issuesListDetail.setPROJECTID(projectId);
                issuesListDetail.setPROJECTNAME(projectName);
                issuesListDetail.setCreateBy(userId);
                issuesListDetail.setUpdateBy(userId);
                issuesListDetail.setINSPECTOR(userId);
                issuesListDetail.setTenantId(tenantId);
                issuesListDetail.setGmtModified(date);
                issuesListDetail.setGmtCreate(date);
                this.issuesListDeatilMapper.insert(issuesListDetail);
            }
            return Result.success("导入成功,条数：" + importDataList.size());
        } catch (Exception e) {
            return Result.fail("导入失败:" + e.getMessage());
        }
    }

}
